<p:library xmlns:p="http://www.w3.org/ns/xproc"
           xmlns:applicant="http://www.loan.org/applicant"
           xmlns:auto="http://www.loan.org/auto"
           xmlns:loan="http://www.loan.org">

    <p:documentation>
        TOC:

        1. is-underage returns true if the applicant
           is underage, false otherwise. "underage" is based
           on both the state the applicant lives in, and 
           his/her age.

        2. is-high-powered-sports-car returns true if the 
           auto is a high-powered sports car, false otherwise.
           "high-powered sports car" is based on the year,
           make, and model of the car.

    </p:documentation>
    
    <p:declare-step type="applicant:is-underage"
                    name="is-underage">

        <p:input port="auto-loan-application" />
        <p:output port="result"/>
            
        <p:choose>
            <p:variable name="state" select="Loan-Application/Applicant/Address/State" />
            <p:variable name="age" select="Loan-Application/Applicant/Age" />

            <p:when test="($state eq 'AK') and (number($age) le 16)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>true</is-underage></p:inline>
                    </p:input>
                    </p:identity>
            </p:when>
            <p:when test="($state eq 'AL') and (number($age) le 18)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>true</is-underage></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="($state eq 'AZ') and (number($age) le 17)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>true</is-underage></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <!-- ... -->
            <p:when test="($state eq 'OH') and (number($age) le 18)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>true</is-underage></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <!-- ... -->
            <p:when test="($state eq 'WY') and (number($age) le 17)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>true</is-underage></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:otherwise>
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-underage>false</is-underage></p:inline>
                    </p:input>
                </p:identity>
            </p:otherwise>
        </p:choose>

    </p:declare-step>
    
    <p:declare-step type="auto:is-high-powered-sports-car"
                    name="is-high-powered-sports-car">
        
        <p:input port="auto-loan-application" />
        <p:output port="result"/>
        
        <p:choose>
            <p:variable name="year" select="Loan-Application/Car/Year" />
            <p:variable name="make" select="Loan-Application/Car/Make" />
            <p:variable name="model" select="Loan-Application/Car/Model" />

            <p:when test="($make eq 'Mazda') and ($model eq 'Miata') and (number($year) gt 2004)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-high-powered-sports-car>true</is-high-powered-sports-car></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="($make eq 'Mazda') and ($model eq 'RX-8') and (number($year) gt 2006)">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-high-powered-sports-car>true</is-high-powered-sports-car></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <!-- ... -->
            <p:otherwise>
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-high-powered-sports-car>false</is-high-powered-sports-car></p:inline>
                    </p:input>
                </p:identity>
                </p:otherwise>
        </p:choose>
        
    </p:declare-step>
    
    <p:declare-step type="loan:is-acceptable"
                    name="is-acceptable">
        
        <p:input port="deciding-factors" />
        <p:output port="result"/>

        
        <p:choose>
            <p:variable name="is-underage" select="deciding-factors/is-underage" />
            <p:variable name="is-high-powered-sports-car" select="deciding-factors/is-high-powered-sports-car" />

            <p:when test="($is-underage='true') and ($is-high-powered-sports-car='true')">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-acceptable>false</is-acceptable></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="($is-underage='true') and ($is-high-powered-sports-car='false')">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-acceptable>true</is-acceptable></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="($is-underage='false') and ($is-high-powered-sports-car='true')">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-acceptable>true</is-acceptable></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="($is-underage='false') and ($is-high-powered-sports-car='false')">
                <p:identity>
                    <p:input port="source">
                        <p:inline><is-acceptable>true</is-acceptable></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
        </p:choose>
        
    </p:declare-step>
    
    <p:declare-step type="loan:decide-on-application"
                    name="decide-on-application">

        <p:documentation>
            Policy: An underage applicant for an auto loan on a   
            high-powered sports cars is not eligible.
        </p:documentation>
        
        <p:input port="auto-loan-application" />

        <p:output port="result"/>

        
        <applicant:is-underage name="is-underage">
        
            <p:input port="auto-loan-application">
                <p:pipe step="decide-on-application" port="auto-loan-application" />
            </p:input>
        
        </applicant:is-underage>
    
        <auto:is-high-powered-sports-car name="is-high-powered-sports-car">
        
            <p:input port="auto-loan-application">
                <p:pipe step="decide-on-application" port="auto-loan-application" />
            </p:input>
        
        </auto:is-high-powered-sports-car>

        <p:identity name="merge">
            <p:input port="source">
                <p:pipe step="is-underage" port="result" />
                <p:pipe step="is-high-powered-sports-car" port="result" />
            </p:input>
        </p:identity>

        <p:wrap-sequence wrapper="deciding-factors" />
    
        <loan:is-acceptable name="is-acceptable" />

        <p:choose>
            <p:when test="is-acceptable='true'">
                <p:identity>
                    <p:input port="source">
                        <p:inline><decision>ACCEPT</decision></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
            <p:when test="is-acceptable='false'">
                <p:identity>
                    <p:input port="source">
                        <p:inline><decision>REJECT</decision></p:inline>
                    </p:input>
                </p:identity>
            </p:when>
        </p:choose>
        
    </p:declare-step>
    
</p:library>